<?php

require_once 'config/include.php';

/**
 *  Class alfapaymentPayment
 * @property-read string $stage_mode
 * @property-read string $userName
 * @property-read string $password
 * @property-read $tax_system
 * @property-read $send_order
 * @property-read $ffd_version
 */
class alfapaymentPayment extends waPayment implements waIPayment
{

    private $currency = ALFA_PAYMENT_CURRENCY;
    private $version = '2.2.0';

    private $order_id;

    public function payment($payment_form_data, $order_data, $auto_submit = false)
    {
        $order = waOrder::factory($order_data);
        if (!in_array($order->currency_id, $this->allowedCurrency())) {
            throw new waException('Ошибка оплаты. Валюта не поддерживается');
        }

        $action_url = $this->getUrl();

        if ($this->stage_mode == 'two-stage') {
            $action_url .= 'registerPreAuth.do';
        } else if ($this->stage_mode == 'one-stage') {
            $action_url .= 'register.do';
        }

        $params = base64_encode(json_encode(array('app_id' => $this->app_id, 'merchant_id' => $this->merchant_id)));

        $returnUrl = $this->getRelayUrl() . '?params=' . $params;

        $currency_id = $order->currency_id;
        $currency = array_search($currency_id, unserialize($this->currency));

        $data = array(
            'userName' => $this->userName,
            'password' => $this->password,
            'orderNumber' => $order->id . '_' . time(),
            'amount' => floor($order->total * 100),
//            'currency' => $currency,
            'returnUrl' => $returnUrl,
            'jsonParams' => json_encode(
                array(
                    'CMS:' => 'Shop-script ' . wa()->getVersion('installer'),
                    'Module-Version: ' => $this->version
                )
            ),
        );

        if (ALFA_ENABLE_FISCALE_OPTIONS == true && $this->send_order) {
            $data['orderBundle'] = $this->getOrderBundle($order_data);
            $data['taxSystem'] = $this->tax_system;
        }

        $response = $this->sendData($action_url, $data);

        if (isset($response['errorCode']) && $response['errorCode']) {
            throw new waPaymentException(ALFA_PAYMENT_MSG_ERROR_MESSAGE . $response['errorMessage']);
        }

        $formUrl = $response['formUrl'];

        $view = wa()->getView();
        $view->assign('form_url', $formUrl);
        $view->assign('auto_submit', $auto_submit);

        $view->assign('loading_message', ALFA_PAYMENT_TEXT_REDIRECT);
        $view->assign('payment_button_text', ALFA_PAYMENT_TEXT_REDIRECT_BUTTON);

        return $view->fetch($this->path . '/templates/payment.html');
    }

    public function allowedCurrency()
    {
        return unserialize($this->currency);
    }

    private function getUrl()
    {
        if ($this->test_mode) {
            return ALFA_TEST_URL;
        } else {
            return ALFA_PROD_URL;
        }
    }

    /**
     * @param $order_data
     * @return string
     * @throws waPaymentException
     * @throws waException
     */
    protected function getOrderBundle($order_data)
    {
        $contact = new waContact($order_data['contact_id']);
        $email = $contact->get('email', 'default');
        $phone = $contact->get('phone', 'default');

        if (!$email && !$phone) {
            $mail = new waMail();
            $email = $mail->getDefaultFrom();
            $email = key($email);
            if (!$email) {
                self::log($this->id, ALFA_PAYMENT_MSG_EMAIL_ERROR);
                throw new waPaymentException(ALFA_PAYMENT_MSG_PAYMENT_ERROR);
            }
        }

        $order_bundle = array(
            'orderCreationDate' => time() * 1000,
            'customerDetails' => array(
                'email' => $email,
                'phone' => $phone,
                'contact' => $contact->getName(),
            ),
            'cartItems' => array(
                'items' => $this->getOrderItems($order_data)
            ),
        );

        $country = $order_data['data']['shipping_address']['country_name'];
        $city = $order_data->data['shipping_address']['city'];
        $post_address = $order_data->data['shipping_address']['street'];

        if ($country && $city && $post_address) {
            $order_bundle['customerDetails']['deliveryInfo'] = array(
                'deliveryType' => $order_data->data['shipping_name'],
                'country' => $country,
                'city' => $city,
                'postAddress' => $post_address,
            );
        }
        return json_encode($order_bundle);
    }

    /**
     * @param $order_data
     * @return array
     * @throws waException
     * @throws waPaymentException
     */
    protected function getOrderItems($order_data)
    {
        $items = array();
        if (is_array($order_data['items'])) {
            foreach ($order_data['items'] as $key => $data) {
                if (!empty($data['tax_included']) && (int)$data['tax_rate'] > 0) {
                    self::log($this->id, sprintf('НДС не включен в цену товара: %s.', var_export($data, true)));
                    throw new waPaymentException('Ошибка платежа. Обратитесь в службу поддержки.');
                }
                $items[] = $this->formalizeItemData($data, $order_data, $key);
            }
        };

        // DELIVERY
        if (!empty($order_data['shipping'])) {
            if (!$order_data->shipping_tax_included && (int)$order_data->shipping_tax_rate > 0) {
                self::log($this->id, sprintf('НДС не включен в стоимость доставки (%s).', $order_data->shipping_name));
                throw new waPaymentException('Ошибка платежа. Обратитесь в службу поддержки.');
            }
            $data = array(
                'name' => (!empty($order_data->shipping_name)) ? $order_data->shipping_name : ALFA_PAYMENT_TEXT_DELIVERY,
                'total' => $order_data->shipping,
                'price' => $order_data->shipping,
                'quantity' => 1,
                'tax_rate' => $order_data->shipping_tax_rate,
            );
            $position = count($items);
            $items[] = $this->formalizeItemData($data, $order_data, $position, true);
        }
        return $items;
    }

    /**
     * @param $data
     * @param $order_data
     * @param $number
     * @param bool $isDelivery
     * @return array
     */
    public function formalizeItemData($data, $order_data, $number, $isDelivery = false)
    {
        if (!empty($data['total_discount'])) {
            $data['total'] = $data['total'] - $data['total_discount'];
            $data['price'] = round($data['price'], 2) - round(ifset($data['discount'], 0.0), 2); //calculate flexible discounts
        }
        $tax_sum = $this->getTaxSum($data['price'], $data['tax_rate']);
        $item_data = array(
            'positionId' => $number,
            'name' => $data['name'],
            'quantity' => array(
                'value' => $data['quantity'],
                'measure' => ALFA_MEASUREMENT_NAME,
            ),
            'itemAmount' => round($data['total'] * 100),
//            'itemCurrency' => $this->getCurrencyISO4217Code($order_data['currency']),
            'itemCode' => $this->app_id . '_order_' . $order_data['id'] . '_' . $order_data['type'] . '_' . rand(1, 100000),
            'tax' => array(
                'taxType' => $this->getTaxType($data['tax_rate']),
                'taxSum' => round($tax_sum * 100, 2),
            ),
            'itemPrice' => round($data['price'] * 100),
        );

        // FFD 1.05 added
        if ($this->ffd_version == 'v105') {

            $attributes = array();
            $attributes[] = array(
                "name" => "paymentMethod",
                "value" => $this->ffd_paymentMethodType
            );
            $attributes[] = array(
                "name" => "paymentObject",
                "value" => ($isDelivery === true) ? 4 : $this->ffd_paymentObjectType
            );
            $item_data['itemAttributes']['attributes'] = $attributes;
        }

        return $item_data;
    }

    /**
     * Calculation of VAT
     * @param $amount
     * @param $tax
     * @return float|int
     */
    protected function getTaxSum($amount, $tax)
    {
        if (!isset($tax)) {
            return 0;
        }
        $vat = ($amount * $tax) / (100 + $tax);
        return $vat;
    }

    /**
     * @param null $tax
     * @return int|mixed
     * @throws waPaymentException
     */
    protected function getTaxType($tax = null)
    {
        $bank_tax_codes = array(
            0 => 1,
            10 => 2,
            18 => 3,
            20 => 6
        );

        if ($tax === null) {
            return 0; //without tax
        }

        $tax = intval($tax);
        if (empty($bank_tax_codes[$tax])) {
            self::log(
                $this->id,
                "Unknown VAT rate: {$tax}. The list of available bets: see bank documentation."
            );
            throw new waPaymentException(ALFA_PAYMENT_MSG_PAYMENT_ERROR);
        }

        return $bank_tax_codes[$tax];
    }

    private function sendData($url, $data)
    {

        if (!extension_loaded('curl') || !function_exists('curl_init')) {
            throw new waException('PHP extension cURL not found');
        }
        if (!($ch = curl_init())) {
            throw new waException('cURL init error');
        }
        if (curl_errno($ch) != 0) {
            throw new waException('cURL error: ' . curl_errno($ch));
        }

        $alfaCurl = curl_init();
        curl_setopt_array($alfaCurl, array(
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => http_build_query($data),
            CURLOPT_HTTPHEADER => array(
                'CMS: Shop-script ' . wa()->getVersion('installer'),
                'Module-Version: ' . $this->version
            ),
            CURLOPT_SSL_VERIFYHOST => false,
            CURLOPT_SSL_VERIFYPEER => false,


        ));
        $response = curl_exec($alfaCurl);

        self::log(preg_replace("/payment$/", "", strtolower(__CLASS__)), "REQUEST: " . $url . "\n" . json_encode($data) . "\nRESPONSE: " . $response);

        curl_close($alfaCurl);

        $app_error = null;

        if (curl_errno($ch) != 0) {
            $app_error = 'cURL error: ' . curl_errno($ch);
        }
        curl_close($ch);
        if ($app_error) {
            throw new waException($app_error);
        }
        if (empty($response)) {
            throw new waException('Empty server response');
        }

        $json = json_decode($response, true);

        if (!is_array($json)) {
            throw new waException(ALFA_PAYMENT_MSG_ERROR_MESSAGE . $response);
        }

        return $json;
    }

    protected function callbackInit($request)
    {
        if (!empty($request['orderId']) && !empty($request['params'])) {
            $params = json_decode(base64_decode($request['params']), true);
            if (!empty($params['app_id']) && !empty($params['merchant_id'])) {
                $this->app_id = $params['app_id'];
                $this->merchant_id = $params['merchant_id'];
            }
            $this->order_id = $request['orderId'];
        } elseif (!empty($request['app_id'])) {
            $this->app_id = $request['app_id'];
        }
        return parent::callbackInit($request);
    }

    protected function callbackHandler($request)
    {

        if (!$this->order_id) {
            throw new waPaymentException('Error. Unknown order ID.');
        }

        $url = $this->getUrl() . 'getOrderStatus.do';

        $params = array(
            'userName' => $this->userName,
            'password' => $this->password,
            'orderId' => $this->order_id,
        );

        $request = $this->sendData($url, $params);
        $transaction_data = $this->formalizeData($request);

        if ($request['ErrorCode'] == 0 && $request['OrderStatus'] == 2) {
            $message = $request['ErrorMessage'];
            $app_payment_method = self::CALLBACK_PAYMENT;
            $transaction_data['state'] = self::STATE_CAPTURED;
            $transaction_data['type'] = self::OPERATION_AUTH_CAPTURE;
            $url = $this->getAdapter()->getBackUrl(waAppPayment::URL_SUCCESS, $transaction_data);

        } elseif ($request['ErrorCode'] == 0 && $request['OrderStatus'] == 1) {
            $message = $request['ErrorMessage'];
            $app_payment_method = self::CALLBACK_PAYMENT;
            $transaction_data['state'] = self::STATE_CAPTURED;
            $transaction_data['type'] = self::OPERATION_AUTH_CAPTURE;
            $url = $this->getAdapter()->getBackUrl(waAppPayment::URL_SUCCESS, $transaction_data);

        } else {
            $message = $request['ErrorMessage'];

            switch ($request['ErrorCode']) {
                case 2:
                    $message = 'Заказ отклонен по причине ошибки в реквизитах платежа.';
                    $app_payment_method = self::CALLBACK_DECLINE;
                    $transaction_data['state'] = self::STATE_DECLINED;
                    $transaction_data['type'] = self::OPERATION_CANCEL;
                    $url = $this->getAdapter()->getBackUrl(waAppPayment::URL_FAIL, $transaction_data);
                    break;

                case 5:
                    $message = 'Ошибка значения параметра запроса.';
                    $app_payment_method = self::CALLBACK_DECLINE;
                    $transaction_data['state'] = self::STATE_DECLINED;
                    $transaction_data['type'] = self::OPERATION_CANCEL;
                    $url = $this->getAdapter()->getBackUrl(waAppPayment::URL_FAIL, $transaction_data);
                    break;

                case 6:
                    $message = 'Незарегистрированный OrderId.';
                    $app_payment_method = self::CALLBACK_DECLINE;
                    $transaction_data['state'] = self::STATE_DECLINED;
                    $transaction_data['type'] = self::OPERATION_CANCEL;
                    $url = $this->getAdapter()->getBackUrl(waAppPayment::URL_FAIL, $transaction_data);
                    break;

                default:
                    $message = $request['ErrorMessage'];
                    $app_payment_method = self::CALLBACK_DECLINE;
                    $transaction_data['state'] = self::STATE_DECLINED;
                    $transaction_data['type'] = self::OPERATION_CANCEL;
                    $url = $this->getAdapter()->getBackUrl(waAppPayment::URL_FAIL, $transaction_data);
                    break;
            }
        }

        $transaction_data = $this->saveTransaction($transaction_data, $request);
        $result = $this->execAppCallback($app_payment_method, $transaction_data);

        // Redirect customer
        if ($request['ErrorCode'] != '0') {
            wa()->getResponse()->redirect($this->getAdapter()->getBackUrl(waAppPayment::URL_FAIL));
        } else {
            wa()->getResponse()->redirect($this->getAdapter()->getBackUrl(waAppPayment::URL_SUCCESS));
        };
    }

    protected function formalizeData($transaction_raw_data)
    {
        $currency_id = $transaction_raw_data['currency'];

        $transaction_data = parent::formalizeData($transaction_raw_data);
        $transaction_data['native_id'] = $this->order_id;
        $order = explode('_', $transaction_raw_data['OrderNumber']);
        $transaction_data['order_id'] = $order[0];

        $arCurrency = unserialize($this->currency);
        $transaction_data['currency_id'] = $arCurrency[$currency_id];
        $transaction_data['amount'] = $transaction_raw_data['Amount'] / 100.0;

        return $transaction_data;
    }

}